@@ -38,6 +38,8 @@
 #include <klistview.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <knewstuff/knewstuff.h>
+#include <knewstuff/entry.h>
 #include <kprocess.h>
 #include <kstandarddirs.h>
 #include <kstandarddirs.h>
@@ -65,6 +67,7 @@
 #include <stdlib.h>
 
 #include <kdebug.h>
+#include <ktexteditor/templateinterface.h>
 //END Includes
 
 //BEGIN plugin + factory stuff
@@ -177,13 +180,23 @@ void KateFileTemplates::updateTemplateDirs(const QString &d)
   QRegExp re( "\\b(\\w+)\\s*=\\s*(.+)(?:\\s+\\w+=|$)" );
   re.setMinimal( true );
 
+  KConfig *config = kapp->config();
+  QStringList hidden;
+  config->readListEntry( "Hidden", hidden, ';' );
+
   for ( QStringList::Iterator it=templates.begin(); it != templates.end(); ++it )
   {
     QFile _f( *it );
     if ( _f.open( IO_ReadOnly ) )
     {
+      QString fname = (*it).section( '/', -1 );
+
+      // skip if hidden
+      if ( hidden.contains( fname ) )
+        continue;
+
       // Read the first line of the file, to get the group/name
-      TemplateInfo *tmp = new TemplateInfo( *it, KURL(*it).fileName(), "Other" );
+      TemplateInfo *tmp = new TemplateInfo( *it, fname, "Other" );
       bool trymore ( true );
       QTextStream stream(&_f);
       while ( trymore )
@@ -227,15 +240,14 @@ void KateFileTemplates::updateTemplateDirs(const QString &d)
 KateFileTemplates::~KateFileTemplates()
 {
   m_acRecentTemplates->saveEntries( kapp->config(), "Recent Templates" );
+  delete m_emailstuff;
+  delete m_user;
 }
 
 void KateFileTemplates::addView(Kate::MainWindow *win)
 {
   PluginView *view = new PluginView ();
 
-//   (void) new KAction( i18n("&Create Template..."), 0,
-//       this, SLOT(slotCreateTemplate()),
-//       view->actionCollection(), "tools_create_template" );
   (void) new KAction( i18n("&Manage Templates..."), 0,
       this, SLOT(slotEditTemplate()),
       view->actionCollection(), "settings_manage_templates" );
@@ -282,8 +294,10 @@ QStringList KateFileTemplates::groups()
 void KateFileTemplates::refreshMenu( PluginView *v )
 {
   QPopupMenu *m = (QPopupMenu*)(((KActionMenu*)(v->actionCollection()->action("file_new_fromtemplate")))->popupMenu());
+
   // clear the menu for templates
   m->clear();
+
   // restore it
   m_actionCollection->action( "file_template_any" )->plug( m );
   m_acRecentTemplates->plug( m );
@@ -355,7 +369,7 @@ void KateFileTemplates::slotOpenTemplate( const KURL &url )
 {
   // check if the file can be opened
   QString tmpfile;
-  QString filename = url.filename();
+  QString filename = url.fileName();
   kdDebug()<<"file: "<<filename<<endl;
   if ( KIO::NetAccess::download( url, tmpfile, 0L ) )
   {
@@ -441,39 +455,6 @@ void KateFileTemplates::slotOpenTemplate( const KURL &url )
 
     uint line, col;
     line = col = 0;
-    if ( isTemplate )
-    {
-      expandMacros( str );
-
-      // try top place the cursor where there is a "^" (and remove that),
-      // and unescape any other "^"s
-      int pos = 0;
-      uint line = 0;
-      uint col = 0;
-      while ( ( ( pos = str.find( "^", pos) ) > -1 ) )
-      {
-        // if it is prefixed "\", remove that and correct index
-        if ( pos > 0 && ( str.at( pos - 1 ) == '\\' ) )
-        {
-          pos -= 1;
-          str.remove( pos, 1 );
-        }
-        // else, this is the cursor position:
-        // find the line and col numbers and remove the "^",
-        // and correct position.
-        else
-        {
-          line = (uint)str.left( pos ).contains( '\n' );
-          col = (uint)(QMAX( 0, pos - str.findRev( '\n', pos ) - 1 ));
-          str.remove( pos, 1 );
-          pos -= 1;
-        }
-      }
-    }
-
-    // paste the result into the document
-    doc->insertText( 0, 0, str );
-    view->setCursorPosition( line, col );
 
     if ( ! isTemplate )
     {
@@ -508,107 +489,19 @@ void KateFileTemplates::slotOpenTemplate( const KURL &url )
     m_acRecentTemplates->addURL( url );
 
     // clean up
-    if ( m_user )
-    {
-      delete m_user;
-      m_user = 0;
-    }
-    if ( m_emailstuff )
-    {
-      delete m_emailstuff;
-      m_emailstuff = 0;
+    delete m_user;
+    m_user = 0;
+    delete m_emailstuff;
+    m_emailstuff = 0;
+    if (isTemplate) {
+	    KTextEditor::TemplateInterface *ti=KTextEditor::templateInterface(doc);
+	    ti->insertTemplateText(0,0,str,QMap<QString,QString>());
     }
+
+
   }
 }
 
-/**
- * @return true if @p macro could be expanded
- * The expanded value is added to @p ret
- */
-bool KateFileTemplates::expandMacro( const QString & macro, QStringList &ret )
-{
-  kdDebug()<<k_funcinfo<<": "<<macro<<endl;
-  QString resultstr;
-  // Date and time
-  if ( macro == "time" )
-    resultstr = QTime::currentTime().toString();
-  else if ( macro == "date" )
-    resultstr = QDate::currentDate().toString();
-  else if ( macro == "datetime" )
-    resultstr = QDateTime::currentDateTime().toString();
-  else if ( macro == "year" )
-    resultstr = QDate::currentDate().toString( "yyyy" );
-  else if ( macro == "month" )
-    resultstr = QDate::currentDate().toString( "MMMM" );
-
-  // date( FORMAT STRING )
-  else if ( macro.startsWith( "date(" ) )
-  {
-    QRegExp re("date\\s*\\(([^)]*)");
-    if ( ( re.search( macro ) ) > -1 )
-      resultstr = QDateTime::currentDateTime().toString( re.cap( 1 ).stripWhiteSpace() );
-  }
-
-  // User: realname, email, organisation
-  // First, try the KDE email settings,
-  // then the password file. For email, eventually return user@host
-  // FIXME is there a canonical way to access the emaildefaults data??
-  else if ( macro == "realname" )
-  {
-    if ( ! m_emailstuff ) m_emailstuff = new KConfig("emaildefaults");
-    m_emailstuff->setGroup("Defaults");
-    QString g = m_emailstuff->readEntry( "Profile", "" ).prepend( "PROFILE_" );
-    m_emailstuff->setGroup( g );
-    resultstr = m_emailstuff->readEntry( "FullName", "" );
-    // if empty, try getting it from the system
-    if ( resultstr.isEmpty() )
-    {
-      if ( ! m_user )
-        m_user = new KUser( KUser::UseRealUserID );
-
-      resultstr = m_user->fullName();
-
-      if ( resultstr.isEmpty() )
-        resultstr = m_user->loginName();
-    }
-  }
-
-  else if ( macro == "username" )
-  {
-    if ( ! m_user )
-      m_user = new KUser( KUser::UseRealUserID );
-
-    resultstr = m_user->loginName();
-  }
-
-  else if ( macro == "email" )
-  {
-    if ( ! m_emailstuff ) m_emailstuff = new KConfig("emaildefaults");
-    m_emailstuff->setGroup("Defaults");
-    QString g = m_emailstuff->readEntry( "Profile", "" ).prepend( "PROFILE_" );
-    m_emailstuff->setGroup( g );
-    resultstr = m_emailstuff->readEntry( "EmailAddress", "" );
-  }
-
-  else if ( macro == "organization" )
-  {
-    if ( ! m_emailstuff ) m_emailstuff = new KConfig("emaildefaults");
-    m_emailstuff->setGroup("Defaults");
-    QString g = m_emailstuff->readEntry( "Profile", "" ).prepend( "PROFILE_" );
-    m_emailstuff->setGroup( g );
-    resultstr = m_emailstuff->readEntry( "Organization", "" );
-  }
-
-  // Anything else is expected to be an environment variable
-  else resultstr = getenv( macro.latin1() );
-
-  resultstr.replace( "^", "\\^" );
-
-  kdDebug()<<"result: "<<resultstr<<endl;
-  ret += resultstr;
-
-  return true;
-}
 
 QWidget *KateFileTemplates::parentWindow()
 {
@@ -631,6 +524,8 @@ void KateFileTemplates::slotCreateTemplate()
 {
   KateTemplateWizard w( parentWindow(), this );
   w.exec();
+
+  updateTemplateDirs();
 }
 
 // Tools for editing the existing templates
@@ -640,7 +535,7 @@ void KateFileTemplates::slotCreateTemplate()
 // * Set the URL to a writable one if required
 void KateFileTemplates::slotEditTemplate()
 {
-  KDialogBase dlg( parentWindow(), "templatemanager", false, i18n("Manage File Templates"), KDialogBase::Ok|KDialogBase::Cancel);
+  KDialogBase dlg( parentWindow(), "templatemanager", false, i18n("Manage File Templates"), KDialogBase::Close);
   dlg.setMainWidget( new KateTemplateManager( this, &dlg ) );
   dlg.exec();
 }
@@ -832,16 +727,18 @@ KateTemplateWizard::KateTemplateWizard( QWidget *parent, KateFileTemplates *kft
   kti = new KateTemplateInfoWidget( this, 0, kft );
   kdDebug()<<"=== Adding template info page at "<<kti<<endl;
   addPage( kti, i18n("Edit Template Properties") );
-  // fill in the author field
-  {
-    QString _s = "%realname";
-    kft->expandMacros( _s );
-    QString _s1 = "%email";
-    kft->expandMacros( _s1 );
-    if ( ! _s1.isEmpty() )
-      _s += " <" + _s1 + ">";
-    kti->leAuthor->setText( _s );
-  }
+  // get liekly values from KTE
+  QMap<QString, QString> map;
+  map[ "fullname" ] = "";
+  map[ "email" ] = "";
+
+  KTextEditor::TemplateInterface::expandMacros( map, parent );
+  QString sFullname = map["fullname"];
+  QString sEmail = map["email"];
+  QString _s = sFullname;
+  if ( ! sEmail.isEmpty() )
+    _s += " <" + sEmail + ">";
+  kti->leAuthor->setText( _s );
 
   // 3) chose a location - either the template directory (default) or
   // a custom location
@@ -896,22 +793,11 @@ KateTemplateWizard::KateTemplateWizard( QWidget *parent, KateFileTemplates *kft
       "template macros.<p>If any of the data below is incorrect or missing, "
       "edit the data in the KDE email information."), page ) );
 
-  sRealname = "%realname";
-  kft->expandMacros( sRealname );
   cbRRealname = new QCheckBox( i18n("Replace full name '%1' with the "
-      "'%fullname' macro").arg( sRealname ), page );
-  cbRRealname->setEnabled( ! sRealname.isEmpty() );
+    "'%{fullname}' macro").arg( sFullname ), page );
+  cbRRealname->setEnabled( ! sFullname.isEmpty() );
   lo->addWidget( cbRRealname );
 
-  sUsername = "%username";
-  kft->expandMacros( sUsername );
-  cbRUsername = new QCheckBox( i18n("Replace user name '%1' with the "
-      "'%username' macro").arg( sUsername ), page );
-  cbRUsername->setEnabled( ! sUsername.isEmpty() );
-  lo->addWidget( cbRUsername );
-
-  sEmail = "%email";
-  kft->expandMacros( sEmail );
   cbREmail = new QCheckBox( i18n("Replace email address '%1' with the "
       "'%email' macro").arg( sEmail ), page);
   cbREmail->setEnabled( ! sEmail.isEmpty() );
@@ -1097,8 +983,18 @@ void KateTemplateWizard::accept()
       }
 
       QTextStream stream(&file);
+      QString ln;
+      bool trymore = true;
       while ( !stream.eof() )
-        tmp += "\n" + stream.readLine();
+      {
+        // skip template headers
+        ln = stream.readLine();
+        if ( trymore && ln.startsWith("katetemplate:") )
+          continue;
+
+        trymore = false;
+        tmp += "\n" + ln;
+      }
 
       file.close();
       KIO::NetAccess::removeTempFile( tmpfile );
@@ -1108,17 +1004,16 @@ void KateTemplateWizard::accept()
     {
       //   3) if the file is not already a template, escape any "%" and "^" in it,
       //      and try do do some replacement of the authors username, name and email.
-      tmp.replace( "%", "%%" );
-      tmp.replace( "^", "\\^" );
+            tmp.replace( QRegExp("%(?=\\{[^}]+\\})"), "\\%" );
+            tmp.replace( QRegExp("\\$(?=\\{[^}]+\\})"), "\\$" );
+      //tmp.replace( "^", "\\^" );
 
-      if ( cbRRealname->isChecked() && ! sRealname.isEmpty() )
-        tmp.replace( sRealname, "%realname" );
+      if ( cbRRealname->isChecked() && ! sFullname.isEmpty() )
+              tmp.replace( sFullname, "%{realname}" );
 
-      if ( cbRUsername->isChecked() && ! sUsername.isEmpty() )
-        tmp.replace( sUsername, "%username" );
 
       if ( cbREmail->isChecked() && ! sEmail.isEmpty() )
-        tmp.replace( sEmail, "%email" );
+              tmp.replace( sEmail, "%{email}" );
     }
 
     str += tmp;
@@ -1169,7 +1064,6 @@ void KateTemplateWizard::accept()
   }
   else if ( cbOpenTemplate->isChecked() )
     kft->application()->activeMainWindow()->viewManager()->openURL( templateUrl );
-
 }
 //END KateTemplateWizard
 
@@ -1185,16 +1079,34 @@ class KateTemplateItem : public KListViewItem
 };
 //END KateTemplateItem
 
+//BEGIN KFTNewStuff
+class KFTNewStuff : public KNewStuff {
+  public:
+    KFTNewStuff( const QString &type, QWidget *parent=0 ) : KNewStuff( type, parent ), m_win( parent ) {};
+    ~KFTNewStuff() {};
+    bool install( const QString &/*filename*/ ) { return true; }
+    bool createUploadFile( const QString &/*filename*/ ) { return false; }
+    QString downloadDestination( KNS::Entry *entry )
+    {
+      QString dir = KGlobal::dirs()->saveLocation( "data", "kate/plugins/katefiletemplates/templates/", true );
+      return dir.append( entry->payload().filename() );
+    }
+
+  private:
+    QWidget *m_win;
+};
+//END KTNewStuff
+
 //BEGIN KateTemplateManager
 KateTemplateManager::KateTemplateManager( KateFileTemplates *kft, QWidget *parent, const char *name )
   : QWidget( parent, name )
   , kft( kft )
 {
-  QGridLayout *lo = new QGridLayout( this, 2, 4 );
+  QGridLayout *lo = new QGridLayout( this, 2, 6 );
   lo->setSpacing( KDialogBase::spacingHint() );
   lvTemplates = new KListView( this );
   lvTemplates->addColumn( i18n("Template") );
-  lo->addMultiCellWidget( lvTemplates, 1, 1, 1, 4 );
+  lo->addMultiCellWidget( lvTemplates, 1, 1, 1, 6 );
   connect( lvTemplates, SIGNAL(selectionChanged()), this, SLOT(slotUpdateState()) );
 
   btnNew = new QPushButton( i18n("New..."), this );
@@ -1209,6 +1121,14 @@ KateTemplateManager::KateTemplateManager( KateFileTemplates *kft, QWidget *paren
   connect( btnRemove, SIGNAL(clicked()), this, SLOT(slotRemoveTemplate()) );
   lo->addWidget( btnRemove, 2, 4 );
 
+  btnUpload = new QPushButton( i18n("Upload..."), this );
+  connect( btnUpload, SIGNAL(clicked()), this, SLOT(slotUpload()) );
+  lo->addWidget( btnUpload, 2, 5 );
+
+  btnDownload = new QPushButton( i18n("Download..."), this );
+  connect( btnDownload, SIGNAL(clicked()), this, SLOT(slotDownload()) );
+  lo->addWidget( btnDownload, 2, 6 );
+
   lo->setColStretch( 1, 1 );
 
   reload();
@@ -1223,7 +1143,8 @@ void KateTemplateManager::apply()
 
 void KateTemplateManager::reload()
 {
-  // add an item for each template
+  lvTemplates->clear();
+
   QDict<KListViewItem> groupitems; // FIXME QMAP
   for ( uint i = 0; i < kft->templates().count(); i++ )
   {
@@ -1246,6 +1167,7 @@ void KateTemplateManager::slotUpdateState()
 
   btnEdit->setEnabled( cool );
   btnRemove->setEnabled( cool );
+  btnUpload->setEnabled( cool );
 }
 
 void KateTemplateManager::slotEditTemplate()
@@ -1259,8 +1181,73 @@ void KateTemplateManager::slotEditTemplate()
 
 void KateTemplateManager::slotRemoveTemplate()
 {
-  // add to a list "remove"
+  KateTemplateItem *item = dynamic_cast<KateTemplateItem*>( lvTemplates->currentItem() );
+  if ( item )
+  {
+    // Find all instances of filename, and try to delete them.
+    // If it fails (there was a global, unwritable instance), add to a
+    // list of removed templates
+    KConfig *config = kapp->config();
+    QString fname = item->templateinfo->filename.section( '/', -1 );
+    QStringList templates = KGlobal::dirs()->findAllResources(
+        "data", fname.prepend( "kate/plugins/katefiletemplates/templates/" ),
+      false,true);
+    int failed = 0;
+    int removed = 0;
+    for ( QStringList::Iterator it=templates.begin(); it!=templates.end(); ++it )
+    {
+      if ( ! QFile::remove(*it) )
+        failed++;
+      else
+        removed++;
+    }
+
+    if ( failed )
+    {
+      config->setGroup( "KateFileTemplates" );
+      QStringList l;
+      config->readListEntry( "Hidden", l, ';' );
+      l << fname;
+      config->writeEntry( "Hidden", l, ';' );
+    }
+
+    // If we removed any files, we should delete a KNewStuff key
+    // for this template, so the template is installable again.
+    // ### This assumes that the knewstuff name is similar to the template name.
+    kdDebug()<<"trying to remove knewstuff key '"<<item->templateinfo->tmplate<<"'"<<endl;
+    config->setGroup("KNewStuffStatus");
+    config->deleteEntry( item->templateinfo->tmplate );
+
+
+    kft->updateTemplateDirs();
+    reload();
+  }
 }
+
+// KNewStuff upload
+void KateTemplateManager::slotUpload()
+{
+  // TODO something nicer, like preparing the meta data from the template info.
+  KateTemplateItem *item = dynamic_cast<KateTemplateItem*>( lvTemplates->currentItem() );
+  if ( item )
+  {
+    KFTNewStuff *ns = new KFTNewStuff( "katefiletemplates/template", this );
+    ns->upload( item->templateinfo->filename, QString::null );
+  }
+}
+
+// KNewStuff download
+void KateTemplateManager::slotDownload()
+{
+  KFTNewStuff *ns = new KFTNewStuff( "katefiletemplates/template", this );
+  ns->download();
+
+  kft->updateTemplateDirs();
+  reload();
+}
+
 //END KateTemplateManager
 
-// kate: indent-spaces off; indent-width 2; replace-tabs on;
+// kate: space-indent on; indent-width 2; replace-tabs on;
+
+#include "filetemplates.moc"
